package com.cultural.heritage.controller.good;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.coupon.CouponAddRequest;
import com.cultural.heritage.model.dto.coupon.CouponQueryRequest;
import com.cultural.heritage.model.dto.coupon.CouponUpdateRequest;
import com.cultural.heritage.model.dto.exchange.ExchangeAddRequest;
import com.cultural.heritage.model.entity.Coupon;
import com.cultural.heritage.model.entity.Exchange;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.entity.UserCoupon;
import com.cultural.heritage.service.good.CouponService;
import com.cultural.heritage.service.good.ExchangeService;
import com.cultural.heritage.service.good.UserCouponService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.xml.transform.Result;
import java.util.List;


@RestController
@RequestMapping("/coupon")
@Slf4j
@Tag(name = "优惠券接口")
public class CouponController {

    @Resource
    private CouponService couponService;


    @Resource
    private UserService userService;


    @Resource
    private UserCouponService userCouponService;


    @Resource
    private ExchangeService exchangeService;


    /**
     * 添加优惠券
     * @param couponAddRequest  优惠券添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    public BaseResponse<Boolean> addCoupon(@RequestBody CouponAddRequest couponAddRequest) {
        if (couponAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Coupon coupon = new Coupon();
        BeanUtils.copyProperties(couponAddRequest, coupon);
        boolean result = couponService.save(coupon);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 更新优惠券
     * @param couponUpdateRequest   优惠券更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    public BaseResponse<Boolean> updateCoupon(@RequestBody CouponUpdateRequest couponUpdateRequest) {
        if (couponUpdateRequest == null || couponUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Coupon coupon = new Coupon();
        BeanUtils.copyProperties(couponUpdateRequest, coupon);
        boolean result = couponService.updateById(coupon);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 删除优惠券
     * @param couponDeleteRequest   优惠券删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    public BaseResponse<Boolean> deleteCoupon(@RequestBody CommonRequest couponDeleteRequest) {
        if (couponDeleteRequest == null || couponDeleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = couponDeleteRequest.getId();
        boolean result = couponService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 查询优惠券列表
     * @param couponQueryRequest 优惠券查询请求体
     * @return 优惠券列表
     */
    @PostMapping("/list/page")
    public BaseResponse<Page<Coupon>> listCouponVOByPage(@RequestBody CouponQueryRequest couponQueryRequest) {
        if (couponQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = couponQueryRequest.getCurrent();
        long pageSize = couponQueryRequest.getPageSize();
        Page<Coupon> page = couponService.page(new Page<>(current, pageSize),
                couponService.getQueryWrapper(couponQueryRequest));
        return ResultUtils.success(page);
    }


    /**
     * 积分兑换优惠券
     * @param exchangeAddRequest  兑换记录添加请求体
     * @return  是否兑换成功
     */
    @PostMapping("/exchange")
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<Boolean> pointsExchangeCoupon(@RequestBody ExchangeAddRequest exchangeAddRequest) {
        if (exchangeAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 更新用户积分
        Long userId = exchangeAddRequest.getUserId();
        Integer requirePoints = exchangeAddRequest.getRequirePoints();
        Integer quantity = exchangeAddRequest.getQuantity();
        if (userId <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User user = userService.getById(userId);
        ThrowUtils.throwIf(user == null, ErrorCode.NOT_FOUND_ERROR);
        int residual = user.getPoints() - requirePoints * quantity;
        if (residual < 0) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "积分不足");
        }
        user.setPoints(residual);
        boolean save = userService.updateById(user);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);

        // 更新优惠券的库存量
        Long couponId = exchangeAddRequest.getCouponId();
        Coupon coupon = couponService.getById(couponId);
        if (coupon.getTotalNum() - quantity < 0) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "优惠券数量不足");
        }
        coupon.setResidueNum(coupon.getTotalNum() - quantity);
        boolean updateCoupon = couponService.updateById(coupon);
        ThrowUtils.throwIf(!updateCoupon, ErrorCode.OPERATION_ERROR);

        // 校验用户限领量是否已达到上限
        Integer currentNum = couponService.verifyIsReachTheUpperLimit(userId, couponId);
        int limitNum = 0;
        if (currentNum == null) {
            limitNum = quantity;
        } else {
            limitNum = currentNum + quantity;
        }
        if ( limitNum > coupon.getLimitNum()) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "用户限领量已达到上限");
        }
        // 插入一条兑换记录
        Exchange exchange = new Exchange();
        BeanUtils.copyProperties(exchangeAddRequest, exchange);
        boolean result = exchangeService.save(exchange);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);

        // 给当前用户添加优惠券
        UserCoupon userCoupon = new UserCoupon();
        BeanUtils.copyProperties(exchangeAddRequest, userCoupon);
        boolean couponResult = userCouponService.save(userCoupon);
        ThrowUtils.throwIf(!couponResult, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "兑换成功");
    }



    /**
     * 用户删除兑换记录
     * @param commonRequest  兑换记录删除请求体
     * @return  是否兑换成功
     */
    @PostMapping("/delRecord")
    public BaseResponse<Boolean> deleteExchangeRecord(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        boolean result = exchangeService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "删除兑换记录成功");
    }


}
